ad_library {
    This is free software distributed under the terms of the GNU General Public
    License.  Full text of the license is available from the GNU Project:
    http://www.fsf.org/copyleft/gpl.html


    Actions for use with dynamic-types in install.xml

    @author kate@xarg.co.uk
    @creation-date 2005-11-05
}

namespace eval install {}
namespace eval install::xml {}
namespace eval install::xml::action {}
namespace eval install::xml::action::form {}

#####################################
# Dynamic Types Install Actions
#

ad_proc -public install::xml::action::create-type { node } {
  Create types and associated attributes from contained definition.
} {
  
  set name [apm_required_attribute_value $node name]
  set pretty_name [apm_required_attribute_value $node pretty-name]
  set pretty_plural [apm_required_attribute_value $node pretty-plural]
  set table_name [apm_required_attribute_value $node table-name]
  set supertype [apm_attribute_value -default "acs_object" $node supertype]
  set id_column [apm_attribute_value -default "XXX" $node id-column]
  set name_method [apm_attribute_value -default "" $node name-method]
 
  set type_spec_list [list pretty_name $pretty_name \
                               pretty_plural $pretty_plural \
                               table_name $table_name \
                               supertype $supertype \
                               id_column $id_column \
                               name_method $name_method]

  set attributes [xml_node_get_children $node]

  set attr_spec_list [list]
  for {set i 0} {$i < [llength $attributes]} {incr i} {
    set attr [lindex $attributes $i]
    
    if {![string equal [xml_node_get_name $attr] attribute]} {
      error "Unknown xml element \"[xml_node_get_name $attr]\""
    }

    set attr_name [apm_required_attribute_value $attr name]
    set datatype [apm_required_attribute_value $attr datatype]
    set pretty_name [apm_required_attribute_value $attr pretty-name]
    set pretty_plural [apm_attribute_value -default "" $attr pretty-plural]
    set default_value [apm_attribute_value -default "" $attr default-value]

    lappend attr_spec_list $attr_name \
                                 [list datatype $datatype \
                                 pretty_name $pretty_name \
                                 pretty_plural $pretty_plural \
                                 default_value $default_value]

  }
  lappend type_spec_list attributes $attr_spec_list

  dtype::spec::create_type [list $name $type_spec_list]
} 

ad_proc -public install::xml::action::create-form { node } {
  Create forms from contained definition.
} {
  set object_type [apm_required_attribute_value $node object-type]

  set form_spec_list [install::xml::action::form::element $node]
  set form_name [lindex $form_spec_list 0]
  set form_spec [lindex $form_spec_list 1]

  lappend form_spec object_type $object_type

  dtype::spec::create_form [list $form_name $form_spec]
}

ad_proc -public install::xml::action::form::element { node } {
  Create elements from contained definition.
} {
  set element_name [apm_attribute_value -default "" $node name]
  set layout [apm_attribute_value -default "" $node layout]
  set widget [apm_attribute_value -default "" $node widget]
  set custom [apm_attribute_value -default "" $node custom]
  set attribute [apm_attribute_value -default "" $node attribute]
  
  if {[string equal $element_name ""]} {
    if {[string equal $attribute ""]} {
        error [_ dynamic-types.node_element_must_have]
    } else {
        set element_name $attribute
    }
  }
  
  set element_spec_list [list]

  foreach name {layout widget custom attribute} {
    if {![string equal [set $name] ""]} {
      lappend element_spec_list $name [set $name]
    }
  }

  set children [xml_node_get_children $node]

  set child_spec_list [list]
  set params_spec_list [list]

  for {set i 0} {$i < [llength $children]} {incr i} {
    set child [lindex $children $i]

    switch -exact -- [xml_node_get_name $child] {
        element {
            set child_spec_list [concat $child_spec_list \
                [install::xml::action::form::element $child]]
        }
        param {
            set params_spec_list [concat $params_spec_list \
                [install::xml::action::form::param $child]]
        }
        default {
            error "Unknown xml tag \"[xml_node_get_name $child]\""
        }
    }
  }

  lappend element_spec_list elements $child_spec_list params $params_spec_list

  return [list $element_name $element_spec_list] 
}

ad_proc -public install::xml::action::form::param { node } {
  Create params from contained definition.
} {
  set name [apm_required_attribute_value $node name]
  set value [apm_required_attribute_value $node value]
  set source [apm_attribute_value -default "" $node source]
  
  set param_spec_list [list value $value]

  if {![string equal $source ""]} {
    lappend param_spec_list source $source
  }

  set children [xml_node_get_children $node]

  set child_spec_list [list]
  for {set j 0} {$j < [llength $children]} {incr j} {
    set child [lindex $children $j]

    if {![string equal [xml_node_get_name $child] param]} {
        error "Unknown xml element \"[xml_node_get_name $child]\""
    }
    
    set child_spec_list [concat $child_spec_list \
        [install::xml::action::form::param $node]]
  }

  lappend param_spec_list params $child_spec_list

  return [list $name $param_spec_list]
}
